/*==========================================================================

----------------------
--- Licensing Info ---
----------------------

Copyright (C) 2023 by 1xAero

This program is free software: you can redistribute it and/or modify
it, share, upload wherever you want without author's permission.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

-------------------
--- Description ---
-------------------

Quad Damage is the powerful artifact from Quake Champions. 
When the player picks up this item, his damage increases by four times. 
Currently supported mods:

   - Half-Life
		
--------------------
--- Requirements ---
--------------------

* Mods: Half-Life 25th Anniversary Update
* Metamod 1.21p38: 
  - https://metamod-p.sourceforge.net/
* AMXX Version 1.10: 
  - https://www.amxmodx.org/downloads-new.php?branch=master

-----------------
--- Changelog ---
-----------------
	
* release: 1.0 (Dec 6 2023)
  - First public release.

* release: 1.1 (Dec 12 2023)
  - Fixed hitsound on dead bodies. Fixed Quad position after spawn.
    Removed DAMAGE_NO check from touch, fixed remove check for quad entity.
    Fixed crashes on entity remove.

* release: 1.2 (Dec 14 2023)
  - Changed MOVETYPE_NONE to MOVETYPE_STEP (fix for Quad gravity and collision)

* release: 1.3 (Dec 21 2023)
  - Fixed bug with quad not being dropped from suicided player or worldkilled.
    Offsets are retrieving now by get_ent_data with Fakemeta automatically.

 * release: 1.4 (Dec 24 2023)
  - Fixed get_ent_data/set pev logerror on invalid weapon entity/dead player.          

===========================================================================*/

#include <amxmodx>
#include <fakemeta>
#include <hamsandwich>

#define PLUGIN "qc_powerup"
#define VERSION "1.2"
#define AUTHOR "1xAero"

#define FALSE 0
#define TRUE 1

#define NULL 0
#define NULLENT -1
#define MAXPLAYERS 32
#define CBASE_WEAPON 4

//Global
#define FN_TASK_DL 1.0

//Player
#define FN_TASK_PL 3.0

//Damage multiplier
#define QUAD_MULTIPLIER 4

//Quad attack sound delay
#define QUAD_ATTACK_SND_DELAY 1.5

//Respawn tick
#define QUAD_SPAWN_TICK 30

//Seconds till disappear
#define QUAD_TIME 30

//Screenfade
#define QUAD_COLOR_R 0
#define QUAD_COLOR_G 0
#define QUAD_COLOR_B 255
#define QUAD_COLOR_A 50

//Player rendering
#define QUAD_PLAYER_R 255
#define QUAD_PLAYER_G 64
#define QUAD_PLAYER_B 235

#define ITEM_KEY 8967

//Sprite vp_parallel_upright
#define _powerup_quad "sprites/powerup/powerup_quad_active.spr"
#define _powerup_quad_off "sprites/powerup/powerup_quad_inactive.spr"

#define _powerup_size 0.25

//Sounds
#define _powerup_pickup "powerup/PowerupQuad.wav"
#define _powerup_player "powerup/Power_Quad_Picked.wav"
#define _powerup_hit "powerup/HitBeepLow.wav"
#define _powerup_spawned "powerup/Power_Quad_Spawned.wav"
#define _powerup_incoming "powerup/Power_Quad_Incoming.wav"
#define _powerup_off "powerup/Quad_Off.wav"
#define _powerup_enemy "powerup/Power_Quad_Enemy.wav"
#define _powerup_team "powerup/Power_Quad_Team.wav"

//Weapons.
new c_weapon_entity[][] = 
{
	"weapon_357",
	"weapon_9mmAR",
	"weapon_9mmhandgun",
	"weapon_crossbow",
	"weapon_egon",
	"weapon_gauss",
	"weapon_rpg",
	"weapon_shotgun",
	"weapon_crowbar",
	"weapon_handgrenade",
	"weapon_hornetgun",
	"weapon_satchel",
	"weapon_tripmine",
	"weapon_snark"		
}

//Non clip weapons
new c_weapon_noclip[][] = 
{
	"weapon_crowbar",
	"weapon_handgrenade",
	"weapon_hornetgun",
	"weapon_satchel",
	"weapon_tripmine",
	"weapon_snark",
	"weapon_egon",
	"weapon_gauss"
}

//Weapons with second mode.
new c_weapon_mode[][] = 
{
	"weapon_9mmAR",
	"weapon_9mmhandgun",
	"weapon_gauss",
	"weapon_shotgun"
}

//Sounds to precache.
new c_powerup_sounds[][] =
{
	_powerup_pickup,
	_powerup_player,
	_powerup_hit,
	_powerup_spawned,
	_powerup_incoming,
	_powerup_off,
	_powerup_enemy,
	_powerup_team	
}

//Classnames
new _powerup_quad_classname[] = "_powerup_quad"
new _powerup_quad_fake_classname[] = "_powerup_fake_quad"

//Vars
new bool: c_powerup_spawned
new bool: c_player_powerup[MAXPLAYERS + 1]
new bool: c_player_powerup_end[MAXPLAYERS + 1]
new Float: c_power_origin[3]
new Float: c_player_powerup_click[MAXPLAYERS + 1]
new c_power_tick_to_spawn
new c_power_tick_to_end 
new c_screenfade

//Task
enum _: TASK_MANAGER (+= 100)
{
	TASK_POWERUP = 5045,
	TASK_POWERUP_END,
	TASK_RENDER
}

public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR);
	
	//TODO: broken Ham_Touch fix for new game build. 
	register_forward(FM_Touch, "Forward_Touch", FALSE);
	
	for(new c_weapon; c_weapon < sizeof c_weapon_entity; c_weapon ++)
		RegisterHam(Ham_Item_PostFrame, c_weapon_entity[c_weapon], "HamF_Item_PostFrame", FALSE);
		
	RegisterHam(Ham_Killed, "player", "HamF_Killed_Post", TRUE);
	RegisterHam(Ham_TakeDamage, "player", "HamF_TakeDamage", FALSE);	
		
	c_screenfade = get_user_msgid("ScreenFade");
	
	remove_task(TASK_POWERUP);
	set_task(FN_TASK_DL, "Function_CreatePowerup", TASK_POWERUP);	
	
	//Spawn global illumination entity once.
	Function_PowerupEntity(NULLENT, _powerup_quad_off, _powerup_quad_fake_classname, _powerup_size);
}

//Precache resources
public plugin_precache()
{
	engfunc(EngFunc_PrecacheModel, _powerup_quad);
	engfunc(EngFunc_PrecacheModel, _powerup_quad_off);
	
	for(new c_sounds; c_sounds < sizeof c_powerup_sounds; c_sounds ++)
		engfunc(EngFunc_PrecacheSound, c_powerup_sounds[c_sounds]);
}

//Reset on entry.
public client_putinserver(iPlayer)
	Function_ResetBool(iPlayer);	

//Check for disconnected.	
public client_disconnected(iPlayer)
{
	//Drop powerup from disconnected player.
	if(c_player_powerup[iPlayer])
	{	
		Function_ResetBool(iPlayer);
		Function_DropPowerup(iPlayer);
	}	
}

//Touch on entity. iPlayer = iTouched.
public Forward_Touch(iEntity, iPlayer)
{
	if(!pev_valid(iEntity))
		return FMRES_IGNORED;
		
	//Pass for player only.	
	if(!is_user_alive(iPlayer))
		return FMRES_IGNORED;	
	
	//Quad powerup.
	if(!equal(Function_GetClassName(iEntity), _powerup_quad_classname))
		return FMRES_IGNORED;
		
	//Skip for invulnerable players.	
	/*if(pev(iPlayer, pev_takedamage) == DAMAGE_NO)
		return FMRES_IGNORED;*/
		
	//console_print(0, "Entity: %s", Function_GetClassName(iEntity));

	//Remove entity.
	Function_RemoveEntity(_powerup_quad_classname);	
	
	//Give Quad.
	Function_SetupPowerup(iPlayer);

	//Set render with a bit delay
	remove_task(iPlayer + TASK_RENDER);
	set_task(FN_TASK_PL, "Function_PlayerRender", iPlayer + TASK_RENDER);
	
	//Send notice to all players.
	for(new pPlayer = 1; pPlayer < MaxClients + 1; pPlayer ++)
	{
		if(pPlayer && pPlayer != iPlayer)
		{
			//Team Deathmatch.
			new iGame_Mode;
			iGame_Mode = get_cvar_num("mp_teamplay");
			
			if(iGame_Mode > NULL)
			{
				static pPlayerTeam[16], pEnemyTeam[16];	
			
				if(hl_get_user_team(pPlayer, pPlayerTeam, charsmax(pPlayerTeam)) != hl_get_user_team(iPlayer, pEnemyTeam, charsmax(pEnemyTeam)))		
					PlaySound(pPlayer, _powerup_enemy);
				else
					PlaySound(pPlayer, _powerup_team);
			}
			
			//Free for all.
			else
				PlaySound(pPlayer, _powerup_enemy);
		}			
	}		
	
	//Set effects.
	Function_SetPowerupEffects(_powerup_quad_fake_classname, NULL/* | EF_BRIGHTFIELD*/);
	
	//Start tick to remove.
	remove_task(TASK_POWERUP_END);
	set_task(FN_TASK_DL, "Function_RemovePowerup", TASK_POWERUP_END);	
	
	return FMRES_IGNORED;
}

//Weapons.
public HamF_Item_PostFrame(iEntity)
{
	//Throws invalid entity.
	if(!pev_valid(iEntity))
		return HAM_IGNORED;

	static iPlayer, iClip, iButton;	
	iPlayer = get_ent_data_entity(iEntity, "CBasePlayerItem", "m_pPlayer");
	iClip = get_ent_data(iEntity, "CBasePlayerWeapon", "m_iClip");
	
	//Again throws logerror.
	if(is_user_alive(iPlayer))
		iButton = pev(iPlayer, pev_button);
			
	//Hack
	if(iButton & IN_ATTACK)
	{
		//Weapons with clip.
		if(iClip > NULL)	
			Function_PowerupAttack(iPlayer);

		//Non clip weapons.
		for(new c_weapon; c_weapon < sizeof c_weapon_noclip; c_weapon ++)
		{
			if(equal(Function_GetClassName(iEntity), c_weapon_noclip[c_weapon]))
				Function_PowerupAttack(iPlayer);
		}
	}

	//Weapons with second mode.
	else if(iButton & IN_ATTACK2)
	{	
		for(new c_weapon; c_weapon < sizeof c_weapon_mode; c_weapon ++)
		{
			if(equal(Function_GetClassName(iEntity), c_weapon_mode[c_weapon]))
				Function_PowerupAttack(iPlayer);
		}
	}
	
	return HAM_IGNORED;
}

//Drop, if player is dead.
public HamF_Killed_Post(iVictim, iKiller, Gib)
{
	if(!is_user_connected(iVictim) /*|| !is_user_connected(iKiller)*/) 
		return;
	
	//Disabled.
	if(c_player_powerup[iVictim])
	{
		Function_PowerupDisable(iVictim);
		
		//Drop if died with powerup.
		Function_DropPowerup(iVictim);
	}	
}

//Damage setup
public HamF_TakeDamage(iVictim, Inflictor, iAttacker, Float: Damage, Damage_Type)
{
	#define HAMSANDWICH_PARAM 4	

	//Safe
	if(!is_user_connected(iVictim) || !is_user_connected(iAttacker)) 
		return HAM_IGNORED;
		
	//Play hitsound.
	if(is_user_alive(iVictim))
		PlaySound(iAttacker, _powerup_hit);		
	
	//Set damage.	
	if(c_player_powerup[iAttacker])
	{	
		//Apply damage.
		Damage = Damage * QUAD_MULTIPLIER;
		SetHamParamFloat(HAMSANDWICH_PARAM, Damage);
	}
	
	return HAM_IGNORED;
}	

//Cycle it.
public Function_CreatePowerup(TASK_POWERUP)
{
	c_power_tick_to_spawn ++;

	//If not spawned
	if(c_power_tick_to_spawn > QUAD_SPAWN_TICK - 1 && !c_powerup_spawned)
	{
		//Remove countdown for new spawned till touch.
		remove_task(TASK_POWERUP_END);

		//Make fake entity invisible till touch.			
		Function_SetPowerupEffects(_powerup_quad_fake_classname, EF_NODRAW);	
	
		//Stop respawn.
		c_powerup_spawned = true;			
	
		//Spawn.
		Function_PowerupEntity(NULLENT,_powerup_quad, _powerup_quad_classname, _powerup_size);
		
		//Send announcement.
		for(new iPlayer = 1; iPlayer < MaxClients + 1; iPlayer ++)
			PlaySound(iPlayer, _powerup_spawned);

		//Reset ticks
		Function_ResetTick();	
	}

	//Incoming Quad.
	else if(c_power_tick_to_spawn == QUAD_SPAWN_TICK - 10 && !c_powerup_spawned)
	{
		for(new iPlayer = 1; iPlayer < MaxClients + 1; iPlayer ++)
			PlaySound(iPlayer, _powerup_incoming);
	}	
	
	//Repeat.
	remove_task(TASK_POWERUP);
	set_task(FN_TASK_DL, "Function_CreatePowerup", TASK_POWERUP);
}

//Loop till end.
public Function_RemovePowerup(TASK_POWERUP_END)
{
	c_power_tick_to_end ++;
	
	//Play off sound.
	if(c_power_tick_to_end > QUAD_TIME - 5 && c_power_tick_to_end < QUAD_TIME + 1 && c_powerup_spawned)
	{
		for(new iPlayer = 1; iPlayer < MaxClients + 1; iPlayer ++)
		{
			if(c_player_powerup[iPlayer])
			{
				PlaySound(iPlayer, _powerup_off);
				
				//Set flickering.				
				if(c_player_powerup_end[iPlayer])
					Function_ScreenFade(iPlayer, (1 << 12) * 2, (1 << 10) * 10, 0x0004, QUAD_COLOR_R, QUAD_COLOR_G, QUAD_COLOR_B, QUAD_COLOR_A);
				else
					Function_ScreenFade(iPlayer, NULL, NULL, 0x0000, 100, 100, 100, 255);

				c_player_powerup_end[iPlayer] = !c_player_powerup_end[iPlayer];		
			}	
		}		
	}	
	
	//Remove it and start new loop if spawned.
	else if(c_power_tick_to_end > QUAD_TIME && c_powerup_spawned)
	{		
		Function_RemoveEntity(_powerup_quad_classname);		
			
		c_powerup_spawned = false;
		Function_ResetTick();			
	
		for(new iPlayer = 1; iPlayer < MaxClients + 1; iPlayer ++)
		{
			if(c_player_powerup[iPlayer])
				Function_PowerupDisable(iPlayer);		
		}
	}
	
	remove_task(TASK_POWERUP_END);
	set_task(FN_TASK_DL, "Function_RemovePowerup", TASK_POWERUP_END);
}

//Set rendering.
public Function_PlayerRender(taskid)
{
	new iPlayer = taskid - TASK_RENDER;

	if(is_user_alive(iPlayer) && c_player_powerup[iPlayer])
		SetRendering(iPlayer, kRenderFxGlowShell, QUAD_PLAYER_R, QUAD_PLAYER_G, QUAD_PLAYER_B, kRenderNormal, NULL);	
}

//Configure powerup.	
stock Function_SetupPowerup(iPlayer)
{
	c_player_powerup[iPlayer] = true;	
	
	//Sounds
	PlaySound(iPlayer, _powerup_pickup);
	PlaySound(iPlayer, _powerup_player);
	
	//Set screenfade.
	Function_ScreenFade(iPlayer, (1 << 12) * 2, (1 << 10) * 10, 0x0004, QUAD_COLOR_R, QUAD_COLOR_G, QUAD_COLOR_B, QUAD_COLOR_A);
}	

//Create entity
stock Function_PowerupEntity(iPlayer = -1, iModel[128], const iClassname[], Float: SpriteSize)
{   
	//Create entity
	new iEntity = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"));
	set_pev(iEntity, pev_classname, iClassname);
	set_pev(iEntity, pev_iuser1, ITEM_KEY);
	
	//Origin.
	new Float: iOrigin[3];
	
	//Map spawn point.
	if(iPlayer < NULL)
	{
		Function_GetMapName(iOrigin);
		c_power_origin = iOrigin; //Float: { 350.0, -3.0, 32.0 };
	}	
	
	//Dropped by player.	
	else
	{	
		pev(iPlayer, pev_origin, iOrigin);
		c_power_origin = iOrigin;
	}	
	
	//Set origin
	engfunc(EngFunc_SetOrigin, iEntity, c_power_origin);	

	//Set sprite on entity
	engfunc(EngFunc_SetModel, iEntity, iModel);
	dllfunc(DLLFunc_Spawn, iEntity);
	
	//Setup entity
	set_pev(iEntity, pev_rendermode, kRenderTransAdd);
	set_pev(iEntity, pev_renderamt, 200.0);
	
	//Not a fake global entity.
	if(!equal(Function_GetClassName(iEntity), _powerup_quad_fake_classname))
		set_pev(iEntity, pev_solid, SOLID_TRIGGER);
		
	//Fake copy if picked up.	
	else
		set_pev(iEntity, pev_solid, SOLID_NOT);
	
	//Quake particles.	
	//set_pev(iEntity, pev_effects, EF_BRIGHTFIELD);
		
	set_pev(iEntity, pev_movetype, MOVETYPE_STEP);
	set_pev(iEntity, pev_gravity, 1.0);
	set_pev(iEntity, pev_scale, SpriteSize);	
	
	//Push	
	engfunc(EngFunc_SetSize, iEntity, Float: { -25.0, -25.0, -35.0 }, Float: { 25.0, 25.0, 35.0 });
	engfunc(EngFunc_DropToFloor, iEntity);
}

//Screenfade
stock Function_ScreenFade(iPlayer, iDuration, iHoldtime, iFadetype, R, G, B, A)
{
	//Disconnected player.
	if(!is_user_connected(iPlayer))
		return;

	message_begin(MSG_ONE, c_screenfade, _, iPlayer);
	write_short(iDuration); //duration
	write_short(iHoldtime); //hold time
	write_short(iFadetype); //fade type
	write_byte(R); //red
	write_byte(G); //green
	write_byte(B); //blue
	write_byte(A); //alpha
	message_end();
}

//RenderFx
stock SetRendering(iEntity, iFx = kRenderFxNone, r = 255, g = 255, b = 255, iRender = kRenderNormal, iAmount = 16)
{
	static Float: iColor[3];
	iColor[0] = float(r);
	iColor[1] = float(g);
	iColor[2] = float(b);

	set_pev(iEntity, pev_renderfx, iFx);
	set_pev(iEntity, pev_rendercolor, iColor);
	set_pev(iEntity, pev_rendermode, iRender);
	set_pev(iEntity, pev_renderamt, float(iAmount));
}

//Play Quad attack sound.
stock Function_PowerupAttack(iPlayer)
{
	//TODO.
	if(c_player_powerup[iPlayer] && c_player_powerup_click[iPlayer] < get_gametime())
	{			
		c_player_powerup_click[iPlayer] = get_gametime() + QUAD_ATTACK_SND_DELAY;
		emit_sound(iPlayer, CHAN_AUTO, _powerup_pickup, VOL_NORM, ATTN_NORM, 0, PITCH_NORM);
	}
}

//Remove Entity.
stock Function_RemoveEntity(const iClassname[])
{
	new iEntity = NULLENT;

	while((iEntity = engfunc(EngFunc_FindEntityByString, iEntity, "classname", iClassname)) > NULL)
	{
		if(pev_valid(iEntity) && pev(iEntity, pev_iuser1) == ITEM_KEY)
			set_pev(iEntity, pev_flags, FL_KILLME);
	}	
}

//Set effects.
stock Function_SetPowerupEffects(const iClassname[], iEffects)
{
	new iEntity = NULLENT;

	while((iEntity = engfunc(EngFunc_FindEntityByString, iEntity, "classname", iClassname)) > NULL)
		set_pev(iEntity, pev_effects, iEffects);
}

//Reset
stock Function_PowerupDisable(iPlayer)
{
	Function_ResetBool(iPlayer);
	
	Function_ScreenFade(iPlayer, NULL, NULL, 0x0000, 100, 100, 100, 255);
	
	SetRendering(iPlayer);
}

//Return entity classname.
stock Function_GetClassName(iEntity)
{
	new iClassname[32];		
	pev(iEntity, pev_classname, iClassname, charsmax(iClassname));
	
	return iClassname;
}

//Drop.
stock Function_DropPowerup(iPlayer)
{
	new iOrigin[3];
	pev(iPlayer, pev_origin, iOrigin);
		
	//Spawn.
	Function_PowerupEntity(iPlayer, _powerup_quad, _powerup_quad_classname, _powerup_size);
}

//Play client sounds
stock PlaySound(iPlayer, const iSound[])
{
	if(equal(iSound[strlen(iSound) - 4], ".mp3"))
	{
		client_cmd(iPlayer, "mp3 stop");
		client_cmd(iPlayer, "mp3 play sound/%s", iSound);
	}
	else
		client_cmd(iPlayer, "spk ^"sound/%s^"", iSound);
}

//Map spawns.
stock Function_GetMapName(Float: iOrigin[3])
{
	new iMapName[32];
	get_mapname(iMapName, 31);

	if(equal(iMapName, "boot_camp"))
		iOrigin = Float: { -942.0, 462.0, 64.0 };
		
	else if(equal(iMapName, "bounce"))
		iOrigin = Float: { -218.0, 911.0, -2239.0 };
		
	else if(equal(iMapName, "contamination"))
		iOrigin = Float: { 350.0, -3.0, 32.0 };		

	else if(equal(iMapName, "crossfire"))
		iOrigin = Float: { -45.0, 772.0, -1781.0 };

	else if(equal(iMapName, "datacore"))
		iOrigin = Float: { -1145.0, 2337.0, 952.0 };

	else if(equal(iMapName, "disposal"))
		iOrigin = Float: { 870.0, 298.0, -95.0 };

	else if(equal(iMapName, "doublecross"))
		iOrigin = Float: { 414.0, -641.0, -1631.0 };

	else if(equal(iMapName, "frenzy"))
		iOrigin = Float: { -383.0, 752.0, 384.0 };

	else if(equal(iMapName, "gasworks"))
		iOrigin = Float: { -323.0, -370.0, 64.0 };	

	else if(equal(iMapName, "lambda_bunker"))
		iOrigin = Float: { 615.0, -516.0, 636.0 };

	else if(equal(iMapName, "pool_party"))
		iOrigin = Float: { -254.0, 315.0, -811.0 };	

	else if(equal(iMapName, "rapidcore"))
		iOrigin = Float: { -1340.0, 87.0, -255.0 };

	else if(equal(iMapName, "rocket_frenzy"))
		iOrigin = Float: { 1186.0, 652.0, -567.0 };

	else if(equal(iMapName, "rustmill"))
		iOrigin = Float: { -2903.0, 2241.0, -347.0 };

	else if(equal(iMapName, "snark_pit"))
		iOrigin = Float: { -281.0, -928.0, -31.0 };

	else if(equal(iMapName, "stalkyard"))
		iOrigin = Float: { 837.0, 19.0, 80.0 };

	else if(equal(iMapName, "subtransit"))
		iOrigin = Float: { -512.0, 125.0, 0.0 };

	else if(equal(iMapName, "undertow"))
		iOrigin = Float: { -1748.0, 0.0, 384.0 };

	else if(equal(iMapName, "xen_dm"))
		iOrigin = Float: { -164.0, 695.0, -1151.0 };

	//Custom maps.
	else if(equal(iMapName, "dm_ominous"))
		iOrigin = Float: { 296.0, 834.0, -251.0 };					
}

//Global setup.
stock Function_ResetTick()
{
	c_power_tick_to_spawn = NULL;
	c_power_tick_to_end = NULL;	
}

//Player bools.
stock Function_ResetBool(iPlayer)
{
	c_player_powerup[iPlayer] = false;
	c_player_powerup_end[iPlayer] = false;
}	

//https://github.com/rtxa/HL-Stocks/blob/master/hlstocks.inc
#define HL_MAX_TEAMS 10
#define HL_MAX_TEAMNAME_LENGTH 16
#define HL_MAX_TEAMLIST_LENGTH 512

stock static teams[HL_MAX_TEAMS][HL_MAX_TEAMNAME_LENGTH];

stock hl_get_user_model(client, model[], len)
	get_user_info(client, "model", model, len);

stock hl_get_user_team(client, team[] = "", len = NULL)
{
	if(hl_get_user_spectator(client))
		return NULL;

	static Float: tdm;

	global_get(glb_teamplay, tdm);

	if(tdm < 1.0) 
		return NULL;

	if(!len) 
		len = HL_MAX_TEAMNAME_LENGTH;

	hl_get_user_model(client, team, len);

	return __get_team_index(team);
}

stock bool: hl_get_user_spectator(client)
{
	if(pev(client, pev_iuser1) || pev(client, pev_iuser2))
		return true;

	return false;
}

stock __get_team_index(const team[])
{
	static teamid, valid, i;
	teamid = NULL;
	valid = NULL;
	i = NULL;

	__count_teams();

	for(i = 0; i < sizeof(teams); i ++)
	{
		teamid++;

		if(equali(teams[i][0], team))
		{
			valid = 1;
			break;
		}
	}

	if(valid)
		return teamid;

	return NULL;
}

stock __count_teams()
{
	if(!teams[0][0])
	{
		new teamlist[HL_MAX_TEAMLIST_LENGTH];

		get_cvar_string("mp_teamlist", teamlist, charsmax(teamlist));
		__explode_teamlist(teams, charsmax(teams[]), teamlist, ';');
	}

	static teamcount;

	if(!teamcount)
	{
		for(new i = 0; i < sizeof(teams); i ++)
		{
			if(teams[i][0])
				teamcount ++;
		}
	}

	return teamcount;
}

stock __explode_teamlist(output[][], size, input[], delimiter)
{
	new nLen, teamname[HL_MAX_TEAMLIST_LENGTH], numTeams;

	while(nLen < strlen(input) && numTeams < HL_MAX_TEAMS)
	{
		strtok(input[nLen], teamname, charsmax(teamname), "", NULL, delimiter);

		nLen += strlen(teamname) + 1;

		if(__get_team_index__(teamname, output, numTeams) < NULL)
		{
			copy(output[numTeams], size, teamname);

			numTeams ++;
		}
	}
}

stock __get_team_index__(const teamname[], team_names[][], numTeams)
{
	for(new i = 0; i < numTeams; i ++)
	{	
		if(equali(team_names[i], teamname))
			return i;
	}
		
	return -1;
}